import cgi
import datetime
import pytz

import workers.conn
import workers.val
import workers.closest
import workers.radar

HTTP200 = "200 OK"
HTTP400 = "400 Bad Request"
HTTP500 = "500 Internal Server Error"

HTML = "text/html"
JSON = "application/json"
XML = "text/xml"

def doConn(args, cfrm = None, cto = None):
  try:
   try:
    frm = (cfrm or args["from"][0]).encode("latin-1").decode("utf-8")
    to = (cto or args["to"][0]).encode("latin-1").decode("utf-8")
   except UnicodeDecodeError:
    frm = cfrm or args["from"][0]
    to = cto or args["to"][0]

   if not frm or not to:
    raise ValueError()

  except Exception:
   content = "<h1>400 Bad Request</h1>\n"
   content += "\"from\"and \"to\" values are required for this type of request.\n"
   return (HTTP400, HTML, content)

  count = args["count"][0] if "count" in args and args["count"] else 6
  date = args["date"][0] if "date" in args and args["date"] else datetime.datetime.strftime(datetime.datetime.now(pytz.timezone("Europe/Vienna")),"%d.%m.%Y")
  time = args["time"][0] if "time" in args and args["time"] else datetime.datetime.strftime(datetime.datetime.now(pytz.timezone("Europe/Vienna")),"%H:%M")
  mode = True if "mode" in args and args["mode"] and args["mode"][0].lower() == "arr" else False
  details = True if "details" in args else False

  try:
   count = int(count)
   if count < 0 or count > 10:
    raise ValueError()
  except:
   content = "<h1>400 Bad Request</h1>\n"
   content += "The \"count\" value must be a value between 0 and 10."
   return (HTTP400, HTML, content)

  try:
   outtime = datetime.datetime.strptime("%s %s" % (date, time), "%d.%m.%Y %H:%M")
  except:
   content = "<h1>400 Bad Request</h1>\n"
   content += "The \"date\" value must be in DD.MM.YYYY format, the \"time\" value must be in HH:MM format."
   return (HTTP400, HTML, content)

  via = list(args["via"]) if "via" in args else None

  if via and len(via) > 3:
   content = "<h1>400 Bad Request</h1>\n"
   content += "It is not possible to route through more than three \"via\" stations."
   return (HTTP400, HTML, content)

  json = "json" in args

  try:
   content = workers.conn.worker(frm, to, count, outtime, mode, details, json, via)
  except Exception as e:
   content = "<h1>500 Internal Server Error</h1>\n"
   if "debug" in args:
    content += str(e)
   return (HTTP500, HTML, content)

  return (HTTP200, JSON if json else XML, content)

def doVal(args, cfrm):
  try:
   try:
    name = (cfrm or args["name"][0]).encode("latin-1").decode("utf-8")
   except UnicodeDecodeError:
    name = cfrm or args["name"][0]

   if not name:
    raise ValueError()

  except:
   content = "<h1>400 Bad Request</h1>\n"
   content += "A \"name\" value is required for this type of request."
   return HTTP400, HTML, content

  json = "json" in args

  try:
   content = workers.val.worker(name, json)

  except Exception as e:
   content = "<h1>500 Internal Server Error</h1>\n"
   if "debug" in args:
    content += str(e).encode()
   return HTTP500, HTML, content

  return HTTP200, JSON if json else XML, content

def doNearby(args):
  try:
   lat = float(args["lat"][0].replace(",", "."))
   lon = float(args["lon"][0].replace(",", "."))

   if (not lat and not lat == float(0)) or (not lon and not lon == float(0)):
    raise ValueError()

  except:
   content = "<h1>400 Bad Request</h1>\n"
   content += "\"lat\" and \"lon\" values are required for this type of request."
   return HTTP400, HTML, content

  distance = args["distance"][0] if "distance" in args and args["distance"] else 1000

  try:
   distance = int(distance)
   if distance < 0 or distance > 10000:
    raise ValueError()
  except:
   content = "<h1>400 Bad Request</h1>\n"
   content += "\"distance\" must be a value between 0 and 10000."
   return HTTP400, HTML, content

  json = "json" in args

  try:
   content = workers.closest.worker(lat, lon, distance, json)

  except Exception as e:
   content = "<h1>500 Internal Server Error</h1>"
   if "debug" in args:
    content += repr(e)
   return HTTP500, HTML, content

  return HTTP200, JSON if json else XML, content

def doRadar(args):
  trains = args["train"] if "train" in args else None
  json = "json" in args

  try:
   content = workers.radar.worker(trains, json)
  except Exception as e:
   content = "<h1>500 Internal Server Error</h1>\n"
   if "debug" in args:
    content += repr(e)
   return HTTP500, HTML, content

  return HTTP200, JSON if json else XML, content

def application(env, re):
 if env["REQUEST_METHOD"] == "POST":
  args = cgi.parse_qs(env['wsgi.input'].readline().decode(), True)
 elif env["REQUEST_METHOD"] == "GET":
  args = cgi.parse_qs(env['QUERY_STRING'], True)
 else:
  re("405 Method Not Allowed", [])
  return

 conn = False
 val = False
 cfrm = None
 cto = None

 split = env["PATH_INFO"].split("/")
 split = [i.strip() for i in split]

 while "" in split:
  split.remove("")

 if len(split) > 2:
  re("400 Bad Request", [])
  yield "<h1>400 Bad Request</h1>".encode()
  yield "Only one (validate) or two (conn) arguments may be passed as path.".encode()
  return

 if len(split) > 0:
  if len(split) == 1:
   val = True
  else:
   conn = True
   cto = split[1].encode("latin-1").decode("utf-8")

  cfrm = split[0].encode("latin-1").decode("utf-8")

 try:
  rtype = "conn" if conn else "val" if val else args["type"][0]
 except:
  re("400 Bad Request", [])
  yield "<h1>400 Bad Request</h1>".encode()
  yield "A request type must be provided.".encode()
  return

 if rtype.lower() in ["conn", "connection"]:
  status, ctype, content = doConn(args, cfrm, cto)
  re(status, [("Content-Type", ctype)])
  yield content.encode()
  return

 elif rtype.lower() in ["val", "validate"]:
  status, ctype, content = doVal(args, cfrm)
  re(status, [("Content-Type", ctype)])
  yield content.encode()
  return

 elif rtype.lower() in ["closest", "close", "near", "nearby"]:
  status, ctype, content = doNearby(args)
  re(status, [("Content-Type", ctype)])
  yield content.encode()
  return

 elif rtype.lower() in ["radar", "live"]:
  status, ctype, content = doRadar(args)
  re(status, [("Content-Type", ctype)])
  yield content.encode()
  return

 else:
  re("400 Bad Request", [])
  yield "<h1>400 Bad Request</h1>".encode()
  yield "The request type you submitted is invalid.".encode()
  return
